unit About;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: About.pas, released 12 September 2000.

The Initial Developer of the Original Code is Mat Ballard.
Portions created by Mat Ballard are Copyright (C) 1999 Mat Ballard.
Portions created by Microsoft are Copyright (C) 1998, 1999 Microsoft Corp.
All Rights Reserved.

Contributor(s): Mat Ballard                 e-mail: mat.ballard@molsci.csiro.au.

Last Modified: 05/25/2000
Current Version: 1.00

You may retrieve the latest version of this file from:

        http://Chemware.hypermart.net/

This work was created with the Project JEDI VCL guidelines:

        http://www.delphi-jedi.org/Jedi:VCLVCL

in mind. 


Purpose:
This form is used to display a standard "About" dialog box with a few extra wrinkles.

Known Issues:
-----------------------------------------------------------------------------}

interface

uses
{$IFDEF VER80} {Delphi 1.0}
WinProcs,
{$ELSE}
Windows, FileInfo,
{$ENDIF}
SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ShellAPI;

type
  TAboutBox = class(TForm)
    Panel1: TPanel;
    ProgramIcon: TImage;
    ProductLabel: TLabel;
    VersionLabel: TLabel;
    CopyrightLabel: TLabel;
    OKButton: TButton;
    ResourcesLabel: TLabel;
    OSLabel: TLabel;
    URLLabel: TLabel;
    CommentsLabel: TLabel;
    CompanyLabel: TLabel;
    EmailLabel: TLabel;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure URLLabelClick(Sender: TObject);
    procedure ProgramIconClick(Sender: TObject);
    procedure EmailLabelClick(Sender: TObject);
  private
    procedure OpenObject(sObjectPath : PChar);
    
  public
    { Public declarations }
  end;

var
  AboutBox: TAboutBox;
  {This is the "About" dialog box that TAboutDlg displays
   when its Execute function is called.}
  {}
  {As well as the properties mentioned elsewhere, it also shows the
   free resources and operating system.}

{$IFDEF VER80} {Delphi 1}
{procedure Register;}
{$ENDIF}

implementation

{$R *.DFM}

{------------------------------------------------------------------------------
    Procedure: TAboutBox.FormCreate
  Description: standard FormCreate event handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets various Properties by querying the OS
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TAboutBox.FormCreate(Sender: TObject);
var
{$IFDEF VER80} {Delphi 1}
  OS_Version: Longint;
  DOS, WIN: Integer;
  DOS_Minor, DOS_Major, WIN_Minor, WIN_Major: Integer;
{$ELSE}
  Memory_Info: TMEMORYSTATUS;
  Version_Info: TOSVERSIONINFO;
  Info: TFileVersionInfo;
{$ENDIF}
begin
{$IFDEF VER80} {Delphi 1}
  ResourcesLabel.Caption :=
    'Free Resources: ' + IntToStr(GetFreeSpace(0) div (1024 * 1024)) + ' M - ' +
    IntToStr(GetFreeSystemResources(0)) + '/' +
    IntToStr(GetFreeSystemResources(1)) + '/' +
    IntToStr(GetFreeSystemResources(2)) + ' %';

  OS_Version := GetVersion;
  DOS := OS_Version div 65536;
  WIN := OS_Version - 65536 * DOS;
  DOS_Major := DOS div 256;
  DOS_Minor := DOS - 256 * DOS_Major;
  WIN_Minor := WIN div 256;
  WIN_Major := WIN - 256 * WIN_Minor;

  OSLabel.Caption :=
    'DOS Version: ' + IntToStr(DOS_Major) + '.' + IntToStr(DOS_Minor) +
    ' / Windows Version: ' + IntToStr(WIN_Major) + '.' + IntToStr(WIN_Minor);
{$ELSE}
  Memory_Info.dwLength := 32;
  GlobalMemoryStatus(Memory_Info);

  {MemoryLabel.Caption := 'Used Memory: ' +
   IntToStr(Memory_Info.dwMemoryLoad) + ' %'; - always zero}
  ResourcesLabel.Caption := 'Total Free RAM: ' +
   FloatToStrF(Memory_Info.dwAvailPhys / 1048576.0, ffFixed, 7, 3) + ' / ' +
   FloatToStrF(Memory_Info.dwTotalPhys / 1048576.0, ffFixed, 7, 3) + ' M';

  Version_Info.dwOSVersionInfoSize := 148;
  GetVersionEx(Version_Info);
  Case Version_Info.dwPlatformId of
    VER_PLATFORM_WIN32_WINDOWS: OSLabel.Caption :=
      'Windows 95 ' + IntToStr(Version_Info.dwMajorVersion) +
      '.' + IntToStr(Version_Info.dwMinorVersion) + ', build ' +
      IntToStr(Version_Info.dwBuildNumber) + ', ' +
      Version_Info.szCSDVersion;

    VER_PLATFORM_WIN32_NT: OSLabel.Caption :=
      'Windows NT ' + IntToStr(Version_Info.dwMajorVersion) +
      '.' + IntToStr(Version_Info.dwMinorVersion) + ', build ' +
      IntToStr(Version_Info.dwBuildNumber) + ', ' +
      Version_Info.szCSDVersion;
    Else
      OSLabel.Caption := 'OS: Damnned if I know !';
  end;

{get version information:}
  Info := TFileVersionInfo.Create(self);
  Info.ExecutableFile := Application.ExeName;
  ProductLabel.Caption := Info.ProductName;
  CompanyLabel.Caption := 'by ' + Info.CompanyName;
  VersionLabel.Caption := 'Version ' + Info.FileVersion;
  CopyrightLabel.Caption := Info.LegalCopyright;
  CommentsLabel.Caption := Info.Comments;
  Info.Free;
{$ENDIF}
end;

{------------------------------------------------------------------------------
    Procedure: TAboutBox.FormShow
  Description: standard FormShow event handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the position on screen
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TAboutBox.FormShow(Sender: TObject);
begin
  Left := (Screen.Width - Width) div 2;
  Top := (Screen.Height - Height) div 2;
end;

{------------------------------------------------------------------------------
    Procedure: TAboutBox.URLLabelClick
  Description: standard Click handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: Fires up the web browser and goes to the specified URL
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TAboutBox.URLLabelClick(Sender: TObject);
var
  TempString : array[0..255] of char;
begin
  StrPCopy(TempString,URLLabel.Caption);
  OpenObject(TempString);
end;

{------------------------------------------------------------------------------
    Procedure: TAboutBox.ProgramIconClick
  Description: standard Click handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: Fires up the web browser and goes to Borland Delphi
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TAboutBox.ProgramIconClick(Sender: TObject);
begin
  OpenObject('http://www.borland.com/delphi/');
end;

{------------------------------------------------------------------------------
    Procedure: TAboutBox.EmailLabelClick
  Description: standard Click handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: Fires up the web browser and goes to Borland Delphi
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TAboutBox.EmailLabelClick(Sender: TObject);
var
  TempString : array[0..255] of char;
begin
  StrPCopy(TempString, 'mailto: ' + EmailLabel.Caption);
  OpenObject(TempString);
end;

{------------------------------------------------------------------------------
    Procedure: TAboutBox.OpenObject
  Description: throws the input sObjectPath to the OS
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: handles web and email addresses 
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TAboutBox.OpenObject(sObjectPath : PChar);
begin
  ShellExecute(0, Nil, sObjectPath, Nil, Nil,
{$IFDEF VER80} {Delphi 1}
    3);  {?SW_SHOW ?}
{$ELSE}
    SW_NORMAL);
{$ENDIF}
end;


end.

