unit Misc;

{$I Misc.Inc}

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: PlotMisc.pas, released 1 July 2000.

The Initial Developer of the Original Code is Mat Ballard.
Portions created by Mat Ballard are Copyright (C) 1999 Mat Ballard.
Portions created by Microsoft are Copyright (C) 1998, 1999 Microsoft Corp.
All Rights Reserved.

Contributor(s): Mat Ballard                 e-mail: mat.ballard@molsci.csiro.au.

Last Modified: 02/25/2000
Current Version: 1.00

You may retrieve the latest version of this file from:

        http://Chemware.hypermart.net/

This work was created with the Project JEDI VCL guidelines:

        http://www.delphi-jedi.org/Jedi:VCLVCL

in mind. 


Purpose:
Collection of miscellaneous routines and type definitions, mostly poor clones
of those from the Math unit in the Pro versions.

Known Issues:
-----------------------------------------------------------------------------}

interface

uses
  classes, graphics,
{$IFDEF WIN}
  WinTypes,
{$ENDIF}
  SysUtils;

type
  pSingle = ^Single;
  pDouble = ^Double;

{dynamic matrix definitions:}
{$IFDEF DELPHI1}
  TIntegerArray = array[0..MaxInt - 1] of Integer;
  TSingleArray = array[0..MaxInt div 2 - 1] of Single;
  TDoubleArray = array[0..MaxInt div 4 - 1] of Double;
{$ELSE}
  TIntegerArray = array[0..MaxInt div SizeOf(Integer) - 1] of Integer;
  TSingleArray = array[0..MaxInt div SizeOf(Single) -1] of Single;
  TDoubleArray = array[0..MaxInt div SizeOf(Double) - 1] of Double;
{$ENDIF}

{NOTE: Multidimensional dynamic arrays DON'T WORK !
  TIntegerMatrix = array[0..0] of array[0..0] of Integer;
  TSingleMatrix = array[0..0] of array[0..0] of Single;
  TDoubleMatrix = array[0..0] of array[0..0] of Double;}

{dynamic matrix definitions - pointers thereto:}
  pIntegerArray = ^TIntegerArray;
  pSingleArray = ^TSingleArray;
  pDoubleArray = ^TDoubleArray;
  {pIntegerMatrix = ^TIntegerMatrix;
  pSingleMatrix = ^TSingleMatrix;
  pDoubleMatrix = ^TDoubleMatrix;}

{$IFNDEF WIN}
type
  TPoint = record
    X: Longint;
    Y: Longint;
  end;
  TRect = record
    case Integer of
      0: (Left, Top, Right, Bottom: Integer);
      1: (TopLeft, BottomRight: TPoint);
  end;
{$ENDIF}

  pPoint = ^TPoint;

  TXYPoint = record
    X: Single;
    Y: Single;
  end;
  pXYPoint = ^TXYPoint;
{$IFDEF DELPHI1}
  TXYArray = array[0..MaxInt div 4 - 1] of TXYPoint;
{$ELSE}
  TXYArray = array[0..MaxInt div SizeOf(Double) - 1] of TXYPoint;
{$ENDIF}
  pXYArray = ^TXYArray;

  TIdentMapEntry = record
    Value: LongInt;
    Name: String;
  end;

  TFileList = class(TStringList)
    private
    protected
    public
      procedure AppendToFile(const FileName: string); virtual;
    published
  end;

  TMemoryStreamEx = class(TMemoryStream)
    private
    protected
    public
      procedure AppendToFile(const FileName: string); virtual;
  end;

  function GetLineLengthFromStream(TheStream: TMemoryStream): Integer;
  function FindStringInStream(TheString: String; TheStream: TMemoryStream): Boolean;
  function CleanString(AString: String; TheChar: Char): String;
  function StrRev(TheStr: String): String;
  function GetWord (var This_Line: String; Delimiter: String): String;
{The GetWord function returns all the characters up to Delimiter in This_Line,
 and removes all characters up to and including Delimiter from ThisLine.}
{}
{This is very useful for extracting comma or tab-seperated strings (numbers)
 from text data.}

  function IndexOfColorValue(Value: TColor): Integer;
  function IndexOfColorName(Name: String): Integer;
  function GetPalerColor(Value: TColor; Brightness: Integer): TColor;

{$IFDEF NO_MATH}
  function Floor(X: Extended): Integer;
  function IntPower(Base: Extended; Exponent: Integer): Extended;
  function Log10(X: Extended): Extended;
  function Power(Base, Exponent: Extended): Extended;
{$ENDIF}

{$IFDEF DELPHI1}
  function GetCurrentDir: String;
{$ENDIF}

const
  CRLF = #13+#10;
  MY_COLORS_MAX = 15;
{The number of MyColors runs from 0..15.}

{MyColors is based on the Colors definition in Graphics.pas,
 restricted the the basic 16 colors, and in a different order
 more suitable for graphs.}
  MyColors: array[0..15] of TIdentMapEntry = (
    (Value: clBlack; Name: 'Black'),
    (Value: clRed; Name: 'Red'),
    (Value: clBlue; Name: 'Blue'),
    (Value: clGreen; Name: 'Green'),
    (Value: clPurple; Name: 'Purple'),
    (Value: clFuchsia; Name: 'Fuchsia'),
    (Value: clAqua; Name: 'Aqua'),
    (Value: clMaroon; Name: 'Maroon'),
    (Value: clOlive; Name: 'Olive'),
    (Value: clNavy; Name: 'Navy'),
    (Value: clTeal; Name: 'Teal'),
    (Value: clGray; Name: 'Gray'),
    (Value: clSilver; Name: 'Silver'),
    (Value: clLime; Name: 'Lime'),
    (Value: clYellow; Name: 'Yellow'),
    (Value: clWhite; Name: 'White'));


implementation

{------------------------------------------------------------------------------
    Procedure: TFileList.AppendToFile
  Description: appends this stringlist to an existing file
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: saving data to disk
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TFileList.AppendToFile(const FileName: string);
var
  Stream: TStream;
begin
  if (FileExists(FileName)) then
  begin
    Stream := TFileStream.Create(FileName, fmOpenReadWrite);
    Stream.Seek(0, soFromEnd);
  end
  else
  begin
    Stream := TFileStream.Create(FileName, fmCreate);
  end;

  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;
{end TFileList ----------------------------------------------------------------}

{------------------------------------------------------------------------------
    Procedure: TMemoryStreamEx.AppendToFile
  Description: appends this MemoryStream to an existing file
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: saving data to disk
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TMemoryStreamEx.AppendToFile(const FileName: string);
var
  Stream: TStream;
begin
  if (FileExists(FileName)) then
  begin
    Stream := TFileStream.Create(FileName, fmOpenReadWrite);
    Stream.Seek(0, soFromEnd);
  end
  else
  begin
    Stream := TFileStream.Create(FileName, fmCreate);
  end;

  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

{end TMemoryStreamEx ----------------------------------------------------------------}

{------------------------------------------------------------------------------
     Function: GetLineLengthFromStream
  Description: gets the length of the line (of text) at TheStream.Position
       Author: Mat Ballard
 Date created: 08/09/2000
Date modified: 08/09/2000 by Mat Ballard
      Purpose: Stream manipulation
 Return Value: the length of the line, up to CRLF
 Known Issues:
 ------------------------------------------------------------------------------}
function GetLineLengthFromStream(TheStream: TMemoryStream): Integer;
var
  pCR,
  pLF: PChar;
  i: Longint;
begin
  pCR := TheStream.Memory;
  Inc(pCR, TheStream.Position);
{default is the entire stream:}
  GetLineLengthFromStream := TheStream.Size - TheStream.Position;
  for i := TheStream.Position to TheStream.Size-1 do
  begin
    if (pCR^ = #13) then
    begin
      pLF := pCR;
      Inc(pLF);
      if (pLF^ = #10) then
      begin
        GetLineLengthFromStream := i - TheStream.Position;
        break;
      end;
    end;
    Inc(pCR);
  end;
end;

{------------------------------------------------------------------------------
     Function: FindStringInStream
  Description: Finds the first occurrence of TheString in TheStream from
               TheStream.Position onwards
       Author: Mat Ballard
 Date created: 08/09/2000
Date modified: 08/09/2000 by Mat Ballard
      Purpose:
 Return Value: TRUE if successful, FALSE otherwise
 Known Issues:
 ------------------------------------------------------------------------------}
function FindStringInStream(TheString: String; TheStream: TMemoryStream): Boolean;
var
  pStart,
  pTheChar: PChar;
  i,
  j: Longint;
  FoundIt: Boolean;
begin
  pStart := TheStream.Memory;
  Inc(pStart, TheStream.Position);
{default is the entire stream:}
  FindStringInStream := FALSE;
  for i := TheStream.Position to TheStream.Size-1 do
  begin
    pTheChar := pStart;
    FoundIt := TRUE;
    for j := 1 to Length(TheString) do
    begin
      if (pTheChar^ <> TheString[j]) then
      begin
        FoundIt := FALSE;
        break;
      end;
      Inc(pTheChar);
    end;

    if (FoundIt) then
    begin
      TheStream.Position := i;
      FindStringInStream := TRUE;
      break;
    end;

    Inc(pStart);
  end;
end;

{------------------------------------------------------------------------------
     Function: CleanString
  Description: removes offending characters from a string
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: menu manipulation
 Return Value: the cleaned string
 Known Issues:
 ------------------------------------------------------------------------------}
function CleanString(AString: String; TheChar: Char): String;
var
  i: Integer;
  NewString: String;
begin
  NewString := '';
  for i := 1 to Length(AString) do
  begin
    if (AString[i] <> TheChar) then
    begin
      NewString := NewString + AString[i];
    end;
  end;
  CleanString := NewString;
end;

{------------------------------------------------------------------------------
     Function: StrRev
  Description: reverses a string
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: string manipulation
 Return Value: the reverse of a string
 Known Issues:
 ------------------------------------------------------------------------------}
function StrRev(TheStr: String): String;
var
  i: Integer;
  RevStr: String;
begin
  RevStr := '';
  for i := Length(TheStr) downto 1 do
  begin
    RevStr := RevStr + TheStr[i];
  end;
  StrRev := RevStr;
end;

{------------------------------------------------------------------------------
     Function: GetWord
  Description: splits a phrase into two at the delimiter
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: string manipulation
 Return Value: the left side
 Known Issues:
 ------------------------------------------------------------------------------}
Function GetWord (var This_Line: String; Delimiter: String): String;
var
  Delimiter_Position: Integer;
begin
  Delimiter_Position := Pos(Delimiter, This_Line);
  If (Delimiter_Position > 0) Then
  begin
    GetWord := Copy(This_Line, 1, Delimiter_Position-1);
    This_Line := Copy(This_Line, Delimiter_Position + Length(Delimiter), Length(This_Line));
  end
  Else
  begin
    GetWord := This_Line;
    This_Line := '';
  end;
end;

{------------------------------------------------------------------------------
     Function: IndexOfColorValue
  Description: gets the index of a color
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: color manipulation
 Return Value: Index of a color
 Known Issues:
 ------------------------------------------------------------------------------}
function IndexOfColorValue(Value: TColor): Integer;
var
  i: Integer;
begin
  IndexOfColorValue := -1;
  for i := 0 to MY_COLORS_MAX do
  begin
    if (MyColors[i].Value = Value) then
    begin
      IndexOfColorValue := i;
      break;
    end;
  end;
end;

{------------------------------------------------------------------------------
     Function: IndexOfColorName
  Description: gets the name of a color
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: color manipulation
 Return Value: string containing the color name
 Known Issues:
 ------------------------------------------------------------------------------}
function IndexOfColorName(Name: String): Integer;
var
  i: Integer;
begin
  IndexOfColorName := -1;
  for i := 0 to MY_COLORS_MAX do
  begin
    if (MyColors[i].Name = Name) then
    begin
      IndexOfColorName := i;
      break;
    end;
  end;
end;

{------------------------------------------------------------------------------
     Function: GetPalerColor
  Description: gets a paler shade of the input color
       Author: Mat Ballard
 Date created: 09/25/2000
Date modified: 09/25/2000 by Mat Ballard
      Purpose: color manipulation
 Return Value: TColor
 Known Issues:
 ------------------------------------------------------------------------------}
function GetPalerColor(Value: TColor; Brightness: Integer): TColor;
var
  iColor,
  iRed,
  iBlue,
  iGreen: Longint;
begin
  iColor := ColorToRGB(Value);

  iRed := (iColor and $000000FF);
  iRed := iRed + Brightness * ($FF-iRed) div 100;

  iGreen := (iColor and $0000FF00) shr 8;
  iGreen := iGreen + Brightness * ($FF-iGreen) div 100;

  iBlue := (iColor and $00FF0000) shr 16;
  iBlue := iBlue + Brightness * ($FF-iBlue) div 100;

  GetPalerColor := TColor(iRed  or (iGreen shl 8) or (iBlue shl 16));
end;


{The following routines are also found in the Math unit in the Pro versions.
 However, the Borland versions are far more efficient.}
{$IFDEF NO_MATH}
function Log10(X: Extended): Extended;
begin
  Result :=  Ln(X) / 2.7182818284590452353602874713527;
end;

function IntPower(Base: Extended; Exponent: Integer): Extended;
begin
  Result := Power(Base, Exponent);
end;

function Power(Base, Exponent: Extended): Extended;
begin
  if Exponent = 0.0 then
    Result := 1.0               { n**0 = 1 }
  else if (Base = 0.0) and (Exponent > 0.0) then
    Result := 0.0               { 0**n = 0, n > 0 }
  else
    Result := Exp(Exponent * Ln(Base))
end;

function Floor(X: Extended): Integer;
begin
  Result := Integer(Trunc(X));
  if Frac(X) < 0 then
    Result := Result - 1;
end;                                              
{$ENDIF}

{$IFDEF DELPHI1}
function GetCurrentDir: String;
var
  ThisDir: String;
begin
  GetDir(0, ThisDir);
end;
{$ENDIF}

end.
