unit NEdit;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: NEdit.pas, released 12 September 2000.

The Initial Developer of the Original Code is Mat Ballard.
Portions created by Mat Ballard are Copyright (C) 1999 Mat Ballard.
Portions created by Microsoft are Copyright (C) 1998, 1999 Microsoft Corp.
All Rights Reserved.

Contributor(s): Mat Ballard                 e-mail: mat.ballard@molsci.csiro.au.

Last Modified: 05/25/2000
Current Version: 1.00

You may retrieve the latest version of this file from:

        http://Chemware.hypermart.net/

This work was created with the Project JEDI VCL guidelines:

        http://www.delphi-jedi.org/Jedi:VCLVCL

in mind. 


Purpose:
Limit user input to various types and ranges of number

Known Issues:
-----------------------------------------------------------------------------}

{$I Misc.inc}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Misc,
  Forms, Dialogs, StdCtrls;

type
  TNumericType = (ntInteger, ntHex, ntFixed, ntCurrency, ntScientific);

  TDataType = (dtInteger, dtCardinal,
    dtShortint, dtSmallint, dtLongint,
{$IFDEF DELPHI2_UP}
    dtInt64,
{$ENDIF}
    dtByte, dtWord,
{$IFDEF DELPHI2_UP}
    dtLongword,
{$ENDIF}
    dtReal, dtReal48, dtSingle, dtDouble, dtExtended, dtComp
{$IFDEF DELPHI2_UP}
    ,dtCurrency
{$ENDIF}
    );

  TNEdit = class(TCustomEdit)
  private
    { Private declarations }
    FDataType: TDataType;
    FMin: Extended;
    FMax: Extended;
    FNumericType: TNumericType;

    function GetReal: Extended;
{$IFDEF DELPHI4_UP}
    function GetInteger: Int64;
{$ELSE}
    function GetInteger: Integer;
{$ENDIF}
{$IFDEF DELPHI2_UP}
    function GetCurrency: Currency;
{$ENDIF}

    procedure SetDataType(Value: TDataType);
    procedure SetNumericType(Value: TNumericType);
    procedure SetReal(Value: Extended);
{$IFNDEF DELPHI4_UP}
    procedure SetInteger(Value: Integer);
{$ELSE}
    procedure SetInteger(Value: Int64);
{$ENDIF}
{$IFDEF DELPHI2_UP}
    procedure SetCurrency(Value: Currency);
{$ENDIF}

  protected
    Procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoExit; override;
  public
    { Public declarations }
    Constructor Create(AOwner:TComponent);override;
    function IsValid: Boolean;
  published
    { Published declarations }
    Property AsReal: Extended read GetReal write SetReal;
{$IFDEF DELPHI4_UP}
    Property AsInteger: Int64 read GetInteger write SetInteger;
{$ELSE}
    Property AsInteger: Integer read GetInteger write SetInteger;
{$ENDIF}
{$IFDEF DELPHI2_UP}
    Property AsCurrency: Currency read GetCurrency write SetCurrency;
{$ENDIF}
    Property DataType: TDataType read FDataType write SetDataType;
    Property Min: Extended read FMin write FMin;
    Property Max: Extended read FMax write FMax;
    Property NumericType: TNumericType read FNumericType write SetNumericType;

{The Custom... properties:}
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property Enabled;
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
{$IFDEF DELPHI2_UP}
{$ENDIF}
{$IFDEF DELPHI3_UP}
{$ENDIF}
{$IFDEF DELPHI4_UP}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property DragMode;
    property ImeMode;
    property ImeName;
    property ParentBiDiMode;
    property PopupMenu;
    property OnEndDock;
    property OnStartDock;
    property OnStartDrag;
{$ENDIF}
{$IFDEF DELPHI5_UP}
{$ENDIF}
  end;

const
  TNEDIT_VERSION = 100;

  NumericTypes: array[TDataType] of string =
    ('Integer', 'Cardinal',
    'Shortint', 'Smallint', 'Longint',
{$IFDEF DELPHI2_UP}
    'Int64',
{$ENDIF}
    'Byte', 'Word',
{$IFDEF DELPHI2_UP}
    'Longword',
{$ENDIF}
    'Real', 'Real48', 'Single', 'Double', 'Extended', 'Comp'
{$IFDEF DELPHI2_UP}
    , 'Currency'
{$ENDIF}
    );

implementation

{------------------------------------------------------------------------------
    Procedure: TNEdit.Create
  Description: standard constructor
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the Text and DataType
 Known Issues:
 ------------------------------------------------------------------------------}
constructor TNEdit.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Text := '0';
  DataType := dtInteger;
end;

{------------------------------------------------------------------------------
    Procedure: TNEdit.KeyDown
  Description: standard KeyDown event handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: examines Key to see if valid
 Known Issues:
 ------------------------------------------------------------------------------}
Procedure TNEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
  KeyOK: Boolean;
  ChrKey: Char;
begin
  KeyOK := FALSE;
  ChrKey := Chr(Key);
  if ((Key = VK_TAB) or (Key = VK_RETURN)) then
  begin
    if (IsValid) then
      KeyOK := TRUE;
  end
  else if ((Key = VK_BACK) or
           (Key = VK_CLEAR) or
           (Key = VK_RIGHT) or
           (Key = VK_DELETE) or
           (Key = VK_LEFT)) then
  begin
    KeyOK := TRUE;
  end
  else if (ChrKey in ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']) then
  begin
{is a basic numeral:}
    KeyOK := TRUE;
  end
  else if ((ChrKey in ['-']) and (FMin < 0)) then
  begin
{is a negative number:}
    KeyOK := TRUE;
  end
  else
  begin
    Case FNumericType of
      ntHex:
        if (ChrKey in ['a', 'b', 'c', 'd', 'e', 'f', 'A', 'B', 'C', 'D', 'E', 'F']) then
          KeyOK := TRUE;
      ntFixed, ntCurrency:
        if (ChrKey = '.') then
          KeyOK := TRUE;
      ntScientific:
        if (ChrKey in ['.', 'e', 'E']) then
          KeyOK := TRUE;
    end;
  end;

  if (not KeyOK) then Key := 0;
{call the TEdit parent function last:}
  inherited KeyDown(Key, Shift);
end;

{------------------------------------------------------------------------------
    Procedure: TNEdit.SetNumericType
  Description: standard property Set procedure
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the NumericType Property
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TNEdit.SetNumericType(Value: TNumericType);
begin
  case FDataType of
    dtInteger .. {$IFDEF DELPHI2_UP}dtLongword{$ELSE}dtWord{$ENDIF}:
      if ((Value = ntInteger) or (Value = ntHex)) then
        FNumericType := Value;
{$IFDEF DELPHI2_UP}
    dtCurrency:
      if (Value = ntCurrency) then
        FNumericType := Value;
{$ENDIF}
  else
    if ((Value = ntFixed) or (Value = ntScientific)) then
      FNumericType := Value;
  end;
end;

{------------------------------------------------------------------------------
    Procedure: TNEdit.SetDataType
  Description: standard property Set procedure
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the DataType Property
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TNEdit.SetDataType(Value: TDataType);
begin
  FDataType := Value;
  case FDataType of
    dtInteger:
      begin
        FMin := Low(Integer);
        FMax := High(Integer);
      end;
    dtCardinal:
      begin
        FMin := Low(Cardinal);
        FMax := High(Cardinal);
      end;
    dtShortint:
      begin
        FMin := Low(Shortint);
        FMax := High(Shortint);
      end;
    dtSmallint:
      begin
        FMin := Low(Smallint);
        FMax := High(Smallint);
      end;
    dtLongint:
      begin
        FMin := Low(Longint);
        FMax := High(Longint);
      end;
{$IFDEF DELPHI4_UP}
    dtInt64:
      begin
        FMin := Low(Int64);
        FMax := High(Int64);
      end;
{$ENDIF}
    dtByte:
      begin
        FMin := Low(Byte);
        FMax := High(Byte);
      end;
    dtWord:
      begin
        FMin := Low(Word);
        FMax := High(Word);
      end;
{$IFDEF DELPHI4_UP}
    dtLongword:
      begin
        FMin := Low(Longword);
        FMax := High(Longword);
      end;
{$ENDIF}

{now the reals:}
    dtReal48:
      begin
        {FMin := 2.9e-39;}
        FMax := 1.7e38;
        FMin := -FMax;
      end;
    dtSingle:
      begin
        {FMin := 1.5e-45;}
        FMax := 3.4e38;
        FMin := -FMax;
      end;
    dtDouble, dtReal:
      begin
        {FMin := 5.0e-324;}
        FMax := 1.7e308;
        FMin := -FMax;
      end;
    dtExtended:
      begin
        {FMin := 3.6e-4951;}
        FMax := 1.1e4932;
        FMin := -FMax;
      end;
    dtComp:
      begin
        FMin := -IntPower(2, 63) + 1;
        FMax := IntPower(2, 63) -1;
      end;
{$IFDEF DELPHI2_UP}
    dtCurrency:
      begin
        FMin := -922337203685477.5808;
        FMax := 922337203685477.5807;
      end;
{$ENDIF}
  end;
  case FDataType of
    dtInteger .. {$IFDEF DELPHI2_UP}dtLongword{$ELSE}dtWord{$ENDIF}:
      if not ((FNumericType = ntInteger) or
              (FNumericType = ntHex)) then
        FNumericType := ntInteger;
{$IFDEF DELPHI2_UP}
    dtCurrency:
      FNumericType := ntCurrency;
{$ENDIF}
    else
      if not ((FNumericType = ntFixed) or
              (FNumericType = ntScientific)) then
        FNumericType := ntScientific;
  end;
end;

{------------------------------------------------------------------------------
     Function: TNEdit.IsValid
  Description: examines Text to determine if it is a valid number
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: see Description
 Return Value: TRUE or FALSE
 Known Issues:
 ------------------------------------------------------------------------------}
function TNEdit.IsValid: Boolean;
var
  RealValue: Extended;
  TheResult: Boolean;
begin
  try
    TheResult := TRUE;
    case FDataType of
      dtInteger .. dtWord:
        begin
          if (FNumericType = ntHex) then
            RealValue := StrToInt('$' + Text)
           else
            RealValue := StrToInt(Text);
        end;
{$IFDEF DELPHI2_UP}
      dtLongword:
        RealValue := StrToInt(Text);
      dtCurrency:
        RealValue := StrToCurr(Text);
{$ENDIF}
    else
      RealValue := StrToFloat(Text);
    end;
    if ((RealValue < FMin) or
        (RealValue > FMax)) then
      TheResult := FALSE;
  except
    TheResult := FALSE;
  end;
  if (not TheResult) then
  begin
    raise EConvertError.CreateFmt(Text + ' is not a valid %s between %g and %g',
      [NumericTypes[FDataType], FMin, FMax]);
  end;    
  IsValid := TheResult;
end;

{------------------------------------------------------------------------------
    Procedure: TNEdit.DoExit
  Description: standard DoExit event handler
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: examines the validity of the Text
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TNEdit.DoExit;
begin
  IsValid;
  inherited DoExit;
end;

{------------------------------------------------------------------------------
     Function: TNEdit.GetReal
  Description: standard property Get function
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: gets the value of the Text Property as a Real
 Return Value: Extended
 Known Issues:
 ------------------------------------------------------------------------------}
function TNEdit.GetReal: Extended;
begin
  try
    GetReal := StrToFloat(Text);
  except
    GetReal := 0;
  end;
end;

{------------------------------------------------------------------------------
     Function: TNEdit.GetInteger
  Description: standard property Get function
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: gets the value of the Text Property as an Integer
 Return Value: Int64
 Known Issues:
 ------------------------------------------------------------------------------}
function TNEdit.GetInteger: {$IFDEF DELPHI4_UP}Int64{$ELSE}Integer{$ENDIF};
begin
  try
    if (FNumericType = ntHex) then
      GetInteger := StrToInt('$' + Text)
     else
      GetInteger := StrToInt(Text);
  except
    GetInteger := 0;
  end;
end;

{------------------------------------------------------------------------------
     Function: TNEdit.GetCurrency
  Description: standard property Get function
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: gets the value of the Text Property as Currency
 Return Value: Currency
 Known Issues:
 ------------------------------------------------------------------------------}
{$IFDEF DELPHI2_UP}
function TNEdit.GetCurrency: Currency;
begin
  try
    GetCurrency := StrToCurr(Text);
  except
    GetCurrency := 0;
  end;
end;
{$ENDIF}

{------------------------------------------------------------------------------
    Procedure: TNEdit.SetReal
  Description: standard property Set procedure
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the Text Property numerically
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TNEdit.SetReal(Value: Extended);
begin
  case FDataType of
    dtReal, dtReal48, dtSingle, dtDouble, dtExtended, dtComp: ;
    else DataType := dtExtended; {this then sets numeric type}
  end;
  Text := FloatToStr(Value);
end;

{------------------------------------------------------------------------------
    Procedure: TNEdit.SetInteger
  Description: standard property Set procedure
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the Text Property numerically
 Known Issues:
 ------------------------------------------------------------------------------}
procedure TNEdit.SetInteger(Value: {$IFDEF DELPHI4_UP}Int64{$ELSE}Integer{$ENDIF});
begin
  case FDataType of
    dtInteger .. {$IFDEF DELPHI2_UP}dtLongword{$ELSE}dtWord{$ENDIF}: ;
    else DataType := dtInteger; {this then sets numeric type}
  end;

  if (FNumericType = ntHex) then
    Text := IntToHex(Value, 0)
   else
    Text := IntToStr(Value);
end;

{------------------------------------------------------------------------------
    Procedure: TNEdit.SetCurrency
  Description: standard property Set procedure
       Author: Mat Ballard
 Date created: 04/25/2000
Date modified: 04/25/2000 by Mat Ballard
      Purpose: sets the Text Property numerically
 Known Issues:
 ------------------------------------------------------------------------------}
{$IFDEF DELPHI2_UP}
procedure TNEdit.SetCurrency(Value: Currency);
begin
  FDataType := dtCurrency;
  Text := CurrToStr(Value);
end;
{$ENDIF}


end.
